package mathClient;

import java.net.*;
import java.io.*;
import java.util.Scanner;

/**
 * A simple client that connects to a MathServer using Sockets and through
 * user input, is able to ask the server to calculate simple math functions.
 * @author Scott Beca (sbeca), 267 362
 * @version 1.0
 * @since 17/03/2011
 */
public class MathClient {
	private static String serverHost;
	private static int serverPort;
	
	/**
	 * Does the user input and sends the math questions to gridbus_rpc().
	 * @param args First argument is a String for the server name.
	 * Second argument is the port number to connect to the server with.
	 */
	public static void main(String[] args) {
		boolean doAnother = true, gotFloat;
		Scanner keyboard = new Scanner(System.in);
		String functionName = null, anotherAnswer = null;
		float functionInput = 0, functionResult;
		
		// Get and check the arguments
		if (args.length != 2) {
			System.out.println("The host or port number were not supplied via"
					+ " a command-line option.");
			return;
		}

		serverHost = args[0];
		try {
			serverPort = Integer.parseInt(args[1]);
		} catch (NumberFormatException e1) {
			System.out.println("Please supply an integer for the port number"
					+ " via a command-line option.");
			return;
		}
		
		System.out.println("Welcome to the Math client.");
		
		// The main loop. Used so the user can ask multiple questions of the
		// server without having to keep opening the program
		while (doAnother)
		{
			System.out.println("Please enter the name of the math function"
					+ " you want to have computed:");
			System.out.print("> ");
			functionName = keyboard.nextLine().split(" ")[0];
			
			// Keep trying to get a valid float
			gotFloat = false;
			while (!gotFloat) {
				System.out.println("Please enter the value to be input into the"
						+ " math function:");
				System.out.print("> ");
				
				try {
					functionInput = keyboard.nextFloat();
					gotFloat = true;
				} catch (Exception e) {
					System.out.println("That input is not a valid float.");
				} finally {
					keyboard.nextLine();
				}
			}
			functionResult = gridbus_rpc(functionName, functionInput);
			 
			// Display result to the user
			if (functionName.compareToIgnoreCase("prime") == 0) {
				System.out.printf("The value of %s(%f) is %b%n",
						functionName, functionInput, (functionResult != 0));
			} else {
				System.out.printf("The value of %s(%f) is %f%n",
						functionName, functionInput, functionResult);
			}
			
			System.out.println("Do you want to calculate another another?"
					+ " (y/n)");
			System.out.print("> ");
			
			anotherAnswer = keyboard.nextLine();
			if (anotherAnswer.length() == 0 || "y".compareToIgnoreCase(
					anotherAnswer.substring(0, 1)) != 0) {
				doAnother = false;
			}
		}
	}

	/**
	 * Makes the connection to the server using Sockets.
	 * Sends the name of the math function and the value to the server
	 * and waits for the answer.
	 * @param function The name of the function to calculate.
	 * @param value The value to be input into the function.
	 * @return The result of the function on the value.
	 */
	private static float gridbus_rpc(String function, float value)
	{
		float result = 0;
		String st = null;
		
		try {
			// Open your connection to a server
			Socket s1 = new Socket(serverHost, serverPort);
			
			OutputStream s1out = s1.getOutputStream();
			DataOutputStream dos = new DataOutputStream (s1out);
			
			// Send the function name and value in string format
			dos.writeUTF(function + " " + value);
			
			// Get an input file handle from the socket and read the input
			InputStream s1In = s1.getInputStream();
			DataInputStream dis = new DataInputStream(s1In);
			
			st = new String (dis.readUTF());
			result = Float.parseFloat(st);
			
			// Close the connection and exit
			dos.close();
			s1out.close();
			dis.close();
			s1In.close();
			s1.close();
			
		} catch (UnknownHostException e) {
			e.printStackTrace();
		} catch (IOException e) {
			System.out.println("There was a problem in the communication to the"
					+ " server. The following result will likely be incorrect.");
		} catch (NumberFormatException e) {
			if (st.compareToIgnoreCase("invalid function") == 0) {
				System.out.println("That math function is not supported by the"
						+ " server.");
				result = 0;
			}
		}

		return result;
	}
}
