package pMathServer;

import java.net.*;
import java.io.*;

/**
 * A simple math server that supplies clients with the answers to some 
 * basic math functions.
 * @author Scott Beca (sbeca), 267 362
 * @version 1.0
 * @since 17/03/2011
 */
public class MathServer {

	/**
	 * The master thread. The server listens on the supplied port number
	 * and starts a new worker thread for any clients that want to connect.
	 * @param args The port number for the server to listen on.
	 */
	public static void main(String[] args) {
		int port;
		
		// Get and check the arguments
		if (args.length != 1)
		{
			System.out.println("A port number was not supplied via a"
					+ " command-line option.");
			return;
		}
		
		try {
			port = Integer.parseInt(args[0]);
		} catch (NumberFormatException e1) {
			System.out.println("Please supply an integer for the port number"
					+ " via a command-line option.");
			return;
		}
		
		System.out.printf("Attempting to start server on port %d now...%n",
				port);
				
		try {
			// Start listening on the supplied port
			ServerSocket s = new ServerSocket(port);
			System.out.println("Server start successful.");

			while(true)
			{
				Socket clientSocket = null;
				// Wait and accept a connection
				clientSocket = s.accept();
				
				// Create a new thread to handle the new client
				new Thread(new MathThread(clientSocket)).start();
			}
		} catch (IOException e) {
			if (e.getClass().equals(java.net.BindException.class)) {
				System.out.println("Port already in use. Please try another"
						+ " one.");
			} else {
				e.printStackTrace();
			}
		}
	}
}
