package pMathServer;

import java.io.*;
import java.net.*;

/**
 * The worker thread that connects with the client and processes their math
 * requests.
 * @author Scott Beca (sbeca), 267 362
 * @version 1.0
 * @since 17/03/2011
 */
public class MathThread implements Runnable {
	private Socket clientSocket = null;
	
	/**
	 * Class constructor.
	 * @param client The client Socket that this thread has been assigned to
	 * take care of.
	 */
	public MathThread(Socket client)
	{
		clientSocket = client;
	}
	
	/**
	 * The method that is called when the MathServer starts this thread.
	 * Gets the input from the client, calls the required math function
	 * (mostly using the built-in Java Math library) and returns the result.
	 */
	public void run()
	{
		String functionName = null, result = null;
		float functionValue;
		InputStream s1In;
		DataInputStream dis;
		OutputStream s1out;
		DataOutputStream dos;
		
		try {
			// Get an input file handle from the socket and read the input
			s1In = clientSocket.getInputStream();
			dis = new DataInputStream(s1In);
			
			String st = new String (dis.readUTF());
			String[] sts = st.split(" ");
			functionName = sts[0];
			functionValue = Float.parseFloat(sts[1]);
			
			// Call the relevant math function
			if (functionName.compareToIgnoreCase("sqrt") == 0) {
				result = Float.toString((float) Math.sqrt(functionValue));
			} else if (functionName.compareToIgnoreCase("sin") == 0) {
				result = Float.toString((float) Math.sin(functionValue));
			} else if (functionName.compareToIgnoreCase("cos") == 0) {
				result = Float.toString((float) Math.cos(functionValue));
			} else if (functionName.compareToIgnoreCase("tan") == 0) {
				result = Float.toString((float) Math.tan(functionValue));
			} else if (functionName.compareToIgnoreCase("log") == 0) {
				result = Float.toString((float) Math.log(functionValue));
			} else if (functionName.compareToIgnoreCase("prime") == 0) {
				result = Float.toString(isPrime((int) functionValue));
			} else {
				result = "invalid function";
			}

			// Get a communication stream associated with the socket
			s1out = clientSocket.getOutputStream();
			dos = new DataOutputStream (s1out);
			
			// Send the result
			dos.writeUTF(result);
			
			// Close the connection
			dis.close();
			s1In.close();
			dos.close();
			s1out.close();
			clientSocket.close();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (NumberFormatException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Checks to see whether the supplied value is prime or not
	 * @param n The number to be checked
	 * @return 1 if prime, 0 if not
	 */
	private static float isPrime(int n) 
	{
		// 2 is prime so check for it first
		if (n == 2) return 1;
	    // check if n is a multiple of 2
	    if (n % 2 == 0 || n < 2) return 0;
	    // if not, then just check the odds
	    for (int i = 3; i * i <= n; i += 2) {
	        if (n % i == 0)
	            return 0;
	    }
	    return 1;
	}
}
